# ABOUTME: Module entry point for python -m ms_fabric_mcp_server.
# ABOUTME: Allows running the server with python -m ms_fabric_mcp_server.
"""Module entry point for python -m ms_fabric_mcp_server."""

from .cli import main

if __name__ == "__main__":
    main()
